/*    AUTOSEN-L Autostart-Nachruestung fuer SENSEO-Latte HD7854     */
/*                       __    _                                    */                     
/*            __________/ /_  (_)________   ___  __  __             */       
/*           / ___/ ___/ __ \/ / ___/ __ \ / _ \/ / / /             */ 
/*          (__  ) /__/ / / / / /  / /_/ //  __/ /_/ /              */ 
/*         /____/\___/_/ /_/_/_/  / .___(_)___/\__,_/               */ 
/*                               /_/                                */ 
/*                                                                  */ 
/* (c) C.Schirp  released for non-commercial use                    */
/*     this header must be distributed with this source code        */ 
/*                                                                  */
/*     debounce routine by peter dannegger                          */
/*                                                                  */
/* V1.0  Erstausgabe (auf Basis autosenseo V1.3)                    */
/*                                                                  */
/*                                                                  */

#include <D:\projekte\picsel\cc5\12F683.h>
//#include <C:\Eigene_Dateien\pic\cc5x\12F683.h>
#include <gpio.h>
//                  FCMEN IESO BODEN1 BODEN0 CPD CP MCLRE PWRTE WDTE FOSC2 FOSC1 FOSC0
//                0 0  . 0     0    0      0  .  1   1   0     1  .  1    1     0     0
#pragma config = 0b.0000.0000.1101.1100



// Defines fuer Status
#define BEGIN	 1
#define	TPRESS1  2
#define	TPRESS2	 3
#define LED_WAIT 4
#define	STABIL	 5
#define BTNPRESSED 6
#define RESTART  7

#define AN 1
#define AUS 0

// Defines fuer Wartezeiten
#define MSWAIT    4  /* Counterstand fr 1ms */
#define BLINKZEIT 1172  /* ca. 1,2s */
#define KLICKZEIT 488   /* ca. 0,5s - 500ms */
#define TASTDRUCK 49    /* ca. 50ms */
#define MAXOFFTIME 1270 /* ca. 1,3 s */
#define MAXLOCKZEIT 4000 /* ca. 5 s */

// Defines fr Tastenspannungen
#define TASTE2_TRESH 51  // 1,0V 84  1,65V - > 84
#define TASTE1_TRESH 204 // 4,0V     3,5V -> 179

// Defines fuer Bit-Test Tastencodes
#define TASTE1	1
#define TASTE2  2


// Variablen
uns8 	Status;		// Zentrale Statusvariable
uns8    key_raw;	// Tasten-Rohwert aus ADC (sonst: GPIO)
uns16	Zeit;		// Universalvariable zum Zeitmessen
uns16   OffZeit;	// Zeit bis Off-Zustand erkannt wird (wird runtergezhlt)
uns16 	OnZeit;		// Zeit bis Aufgeheizt-Zustand erkannt wird (wird runtergezhlt)
uns16	LockZeit;	// Zeit bis Lock freigegeben wird
uns8	LED_referenz:1;	// Referenzmerker fr die LED-Flankenerkennung
uns8	Taste1m:1;	// Merker fr Taste 1
uns8	Taste2m:1;	// Merker fr Taste 1
uns8	Tmerker:1;	// Umschaltmerker T1/T2
uns8	Tester1:1;	// Hilfsvariable 1
uns8	Tester2:1;	// Hilfsvariable 1
uns8	OldLED:1;	// Merker fuer alten LED-Status zur Flankenerkennung
uns8	Lock:1;		// Autostart-Lock fr MAXLOCKZEIT

// Variablen fr debouncing
uns8 ct0, ct1;
uns8 i;
uns8 key_state;
uns8 key_press;


// HW-Mapping
#define AKTIV_port GP5  /* LED am Aktiv-Port blinkt antiparallel zur Power-LED, wenn Autostart aktiv */
#define	LED_port GP3	/* GP3 kann nur als Eingang benutzt werden */

void init(void)
{
	clrwdt();

	OSCCON = 0b.0110.0000;	// interne clock 4MHz

	OPTION = 0b.1101.0111;	// kein weak PU, TM0 in Timer-Mode, Prescaler zum Timer geschaltet
	
	ANSEL  = 0b.0101.0011;  // ADCS = 101, AN0&AN1 analoge Pins

	AKTIV_port = AUS;	// Autostart-LED sofort aus

	TRISIO = 0b.0000.1111; // 0-4 input, 5 Output
	
	// Entprellung zurcksetzen
	key_state = 0xFF;
	ct0 = key_state;
	ct1 = ct0;
	
	// Variablen
	Status = RESTART;
	OldLED = 0;
	OnZeit = 0;
	LockZeit = 0;
	Lock = 0;
	OffZeit = 0;
}

// Debouncing von Peter Dannegger
void debounce(void)
{

  i = key_state ^ key_raw;	// key changed ? (active high)

  ct0 = ~( ct0 & i );		// reset or count ct0
  ct1 = ct0 ^ (ct1 & i);	// reset or count ct1
  i &= ct0 & ct1;		// count until roll over
  key_state ^= i;		// then toggle debounced state
  key_press |= key_state & i;	// 0->1: key pressing detect
}

// Auf Taste testen, wenn druecken erkannt Status auf chstate aendern
void Taste_testen(uns8 chstate)
{
	if (key_press & TASTE1)
	{
		Taste1m = 1;		// merken: Taste 1 war es
		Status = chstate;	// Sollstatus uebernehmen
		key_press ^= TASTE1;	// und den Tastendruck loeschen
	}
	else
	{
		Taste1m = 0;
	}
	
	if (key_press & TASTE2)
	{
		Taste2m = 1;		// merken: Taste 2 war es
		Status = chstate;	// Sollstatus uebernehmen
		key_press ^= TASTE2;	// und den Tastendruck loeschen
	}
	else
	{
		Taste2m = 0;
	}	
}


  
//
//  Taste mit Schwellwerten analog messen und Ergebnis in key_raw eintragen
//
void tasten_messen(void)
{
	key_raw = 0;		// keine Taste gedrckt	
		
	ADCON0=	 0b0000.0001;	// AD-Wandler linkssbuendig, VDD=ref,  Channel AN0, Stopped, AD_ON
	GO = 1;			// Wandlung starten
	
	while (GO);		// Auf ADC warten
	
	if (ADRESH > TASTE1_TRESH) // wir nehmen nur 8 Bit Auflsung, Vergleich mit Treshold
	{
		key_raw = TASTE1;	// Taste gedrckt
	}

		
	ADCON0=	 0b0000.0101;	// AD-Wandler linkssbuendig, VDD=ref,  Channel AN1, Stopped, AD_ON
	GO = 1;			// Wandlung starten
	
	while (GO);		// Auf ADC warten
	
	if (ADRESH < TASTE2_TRESH) // wir nehmen nur 8 Bit Auflsung, Vergleich mit Treshold
	{
		key_raw += TASTE2;	// Taste gedrckt
	}
		
}



// Zeitmessung resetten
void Uhr_starten(void)
{
	TMR0=0;
	Zeit=0;
}

// rund 0,001024s warten (1MHz clock / 256 Vorteiler / Zhlerstand 4)
void wait1ms(void)
{
	TMR0 = 0;
	while (TMR0 < MSWAIT);
		
}


// Test auf "Off-Zustand"
void OffTest(void)
{

	if (LED_port)
	{
		// LED an, dann Uhr aufziehen
		OffZeit	= MAXOFFTIME;
			
	}
	else if (OffZeit > 0)
	{
		// LED ist zwar aus, aber Zeit noch nicht abgelaufen
		OffZeit--;
	}
	else
	{
		Status = RESTART; // Off-Zustand erkannt: immer im RESET-Status hngenbleiben
	}
	
}

// Test auf Lock-Zustand
// Wenn die LED lnger als MAXOFFTIME eingeschaltet ist, wird Lock gesetzt und LockZeit auf Maxlockzeit aufgezogen
// erst wenn die LED mal wieder aus ist, wird der OnZeit-Timer mal wieder gestartet & Lockzeit nicht mehr auf Max gesetzt,
// so dass der Lockzeit-Timer abluft und das Lock wieder freigegeben wird.
void LockTest(void)
{

	if (LED_port)
	{
		// wenn vorher aus war, Zeitmessung starten
		if (!OldLED)
		{
			OnZeit = MAXOFFTIME;
		}
		else if (OnZeit > 0)
		{
			// 
			OnZeit--;
		}
		else // LED ist laenger als 1,3 s an -> Autostart sperren
		{
			Lock = 1;
			LockZeit = MAXLOCKZEIT;
		}
			
	}
	else // LED ist  aus
	{
		// egal
	}
	
	if (LockZeit > 0)
	{
		// 
		LockZeit--;
	}
	else // Lockzeit ist abgelaufen, Lock freigeben
	{
		Lock = 0;
	}
	
	// alten Zustand merken
	OldLED = LED_port;
	
}


// Tastenemulation betaetigen 

void Taste_an(void)
{
	// Je nach Taste 1/2 Puls setzen	
	if (Tmerker)
	{		
		GP1 = 0;// Low anlegen ist fuer T2 aktiv
		ANS1 = 0; // Port auf Digital umschalten
		TRISIO1 = 0;// Port auf Ausgang schalten		
	}
	else
	{		
		GP0 = 1;  // High anlegen ist fuer T1 aktiv
		ANS0 = 0; // Port auf Digital umschalten
		TRISIO0 = 0;// Port auf Ausgang schalten		
	}
}

// Tastenemulation wieder auf nicht bettigt

void Taste_aus(void)
{
	TRISIO = 0b.0000.1111; // Ports auf Input schalten: 0-4 input, 5 Output	
	ANSEL  = 0b.0101.0011;  // Ports auf analog schalten ADCS = 101, AN0&AN1 analoge Pins
}


void main (void)
{
	
	init();
	
	while (1)
	{
		clrwdt();
		// i/O einlesen
		tasten_messen();
		debounce();
		// 1ms-Takt
		wait1ms();
		// Uhr weiterzhlen
		Zeit++;	
		// Aus-Zustand immer abprfen
		OffTest();
		// Lockzustand prfen
		LockTest();
		
		if (Status == BEGIN)
		{
			// Tastendruck testen, wenn nicht gelockt
			if (!Lock)
			{
				Taste_testen(TPRESS1);
			}
			
			// wenn Tastendruck T1 erkannt, Uhr starten, Taste merken
			if (Taste1m)
			{
				Uhr_starten();
				Tmerker = 0;
			}
			
			// wenn Tastendruck T2 erkannt, Uhr starten, Taste merken
			else if (Taste2m)
			{
				Uhr_starten();
				Tmerker = 1;
			}
		}
		// Zustand nach Taste 1x gedrckt
		else if (Status == TPRESS1)
		{
			// Tastendruck testen 
			Taste_testen(TPRESS1);
			
			// Bitvariablen ausrechnen (Compilerbeschraenkung, geht nicht im if() )
			Tester1 = !Tmerker;
			Tester1 &= Taste1m;
			Tester2 = Taste2m;
			Tester2 &= Tmerker;
			
			// wenn gleicher Tastendruck erkannt, nchster State
			if (Tester1)
			{
				Status = TPRESS2;
			}
			else if (Tester2)
			{
				Status = TPRESS2;
			}
			
			// wenn die Doppelklickzeit abgelaufen ist, wieder alles auf Anfang
			else if (Zeit > KLICKZEIT)
			{
				Status = RESTART;
			}

		}
		// Zustand nach Taste 2x gedrckt,  LED-Signal samplen
		if (Status == TPRESS2)
		{		
			LED_referenz=LED_port;
			Uhr_starten();
			Status = LED_WAIT;
			// nach Doppelclick nochmal Extrazeit wegen Bug in der HD7810/69/A
			OffZeit	= MAXOFFTIME;
		}

		//warten auf stabiles LED-Signal		
		if (Status == LED_WAIT)
		{
			// erneuter Tastendruck bricht ab
			Taste_testen(RESTART);
			
			// Aktiv-LED blinken lassen
			if (LED_port)
			{
				AKTIV_port = AUS;
			}
			else
			{
				AKTIV_port = AN;
			}
			
			// wenn sich der LED-Zustand gendert hat, neu starten
			if (LED_referenz != LED_port)
			{
				LED_referenz=LED_port;
				Uhr_starten();
			}	
			
			// Zeit abgelaufen, lnger als 1,2s stabil, unsere LED aus
			if (Zeit > BLINKZEIT)
			{
				Status = STABIL;
				AKTIV_port = AUS;
			}
			
		}
		// wir haben einen stabilen LED-Zustand!
		if (Status == STABIL)
		{
			// Wenn LED an ist, Knopf drcken!
			if (LED_port == AN)
			{
				Taste_an();
				
				Uhr_starten();
				
				Status = BTNPRESSED;
			}
			else
			{
				Status = RESTART;
			}
		}
		
		if (Status == BTNPRESSED)
		{
			// wenn Taste lang genug gedrckt -> alles auf Anfang
			if (Zeit > TASTDRUCK)
			{
				Taste_aus();
				
				Status = RESTART;	
			}
		}
		
		// alles auf Anfang setzen
		if (Status == RESTART)
		{
			Taste1m = 0; /* beide Tastenmerker zuruecksetzen */
			Taste2m = 0;
			Status = BEGIN;
			AKTIV_port = AUS;
		}
	}; // von while(1)

} // von main()	